/*
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */
package to.tetramorph.almanac;
import java.util.Calendar;
/**
 * ボイドタイムを表現するクラスで、ボイド開始と終了時刻、
 * 終了したときの月のサイン(イングレスサイン)からなる。
 */
public class VoidTime {

    public int ingSign;
    public Calendar begin;
    public Calendar end;

    /**
     * ボイドタイムを表現するオブジェクトを作成する。
     * @param begin ボイド開始時刻
     * @param end ボイド終了時刻
     * @param ingSign イングレスサイン番号(0-11)
     */
    public VoidTime(Calendar begin, Calendar end, int ingSign) {
        this.begin = begin;
        this.end = end;
        this.ingSign = ingSign;
    }

    /** ボイドの開始時刻と終了時刻、イングレスサイン番号文字列を返す。*/
    @Override
    public String toString() {
        return String.format("%tY/%tm/%td %tT", begin, begin, begin, begin)
                + "  "
                + String.format("%tY/%tm/%td %tT", end, end, end, end)
                + "  " + ingSign;
    }

    /**
     * 24時間制でボイド開始時刻の文字列を返す。"0:00"〜"23:59"といった表現
     */
    public String getBeginTime() {
        return String.format("%Tk:%TM", begin, begin, begin);
    }

    /**
     * 24時間制でボイド終了時刻の文字列を返す。"0:00"〜"23:59"といった表現
     */
    public String getEndTime() {
        return String.format("%Tk:%TM", end, end, end);
    }

    /**
     * イングレスサインの前のサイン番号(0-11)を返す。
     */
    public int getPrevSign() {
        int temp = ingSign - 1;
        if (temp < 0) {
            temp = 11;
        }
        return temp;
    }
}