/*
 * CalendarMake.java
 *
 * Created on 2008/01/19, 18:52
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.kyuureki;

import java.util.Calendar;
import java.util.GregorianCalendar;

/**
 * 干支カレンダー生成のテスト。
 * @author 大澤義鷹
 */
class CalendarMake {
    static final String [] youbi = { "無効","日","月","火","水","木","金","土" };

    /**  CalendarMake オブジェクトを作成する */
    public CalendarMake() {
    }
    /**
     * GregorianCalendarの日付文字列を返す。
     * "yyyy-mm-dd,曜日,hh:mm"という書式で返す。
     */
    static String getDate(GregorianCalendar cal) {
        //四捨五入する際カレンダーの値を変更してしまうのでクローンを作成
        GregorianCalendar gcal = (GregorianCalendar)cal.clone();
        //0.5秒以上は切り上げ、以下なら切り捨て
        if(gcal.get(Calendar.MILLISECOND) >= 500) gcal.add(Calendar.SECOND,1);
        gcal.set(Calendar.MILLISECOND,0);
        //30秒以上は切り上げ、以下なら切り捨て。
        if(gcal.get(Calendar.SECOND) >= 30) gcal.add(Calendar.MINUTE,1);
        gcal.set(Calendar.SECOND,0);
        String w = youbi[gcal.get(Calendar.DAY_OF_WEEK)];
        int minute = gcal.get(Calendar.MINUTE);
        int hour = gcal.get(Calendar.HOUR_OF_DAY);
        //return String.format("%tY-%tm-%td,%s,%02d:%02d",gcal,gcal,gcal,w,hour,minute);
        return String.format("%tY-%tm-%td,%s,",gcal,gcal,gcal,w);
    }
    /**
     * 1916年から8400日間の干支カレンダーを出力
     */
    static void test1() {
        GregorianCalendar cal = new GregorianCalendar(1744,5-1,1,0,0,0);
        //EasternEphemeris ee = new EasternEphemeris();
        Kansi kansi = new Kansi();
        for(int i=0; i<365; i++) {
            String date = getDate(cal);
            KansiKyuuseiAlmanac.getValue(cal,kansi);
            String k = kansi.get干支(Kansi.DAY);
            System.out.println(date + k);
            cal.add(Calendar.DAY_OF_MONTH,1);
        }        
    }
    static void test2() {
        GregorianCalendar cal = new GregorianCalendar(1905,11-1,21,0,0,0);
        //EasternEphemeris ee = new EasternEphemeris();
        Kansi kansi = new Kansi();
        for(int i=0; i<40; i++) {
            String date = getDate(cal);
            //ee.getKansi(cal,kansi);
            KansiKyuuseiAlmanac.getValue(cal,kansi);
            String y = kansi.get干支(Kansi.YEAR);
            String m = kansi.get干支(Kansi.MONTH);
            String k = kansi.get干支(Kansi.DAY);
            System.out.println(date + y + "," + m + "," + k);
            cal.add(Calendar.DAY_OF_MONTH,-4200);
        }
        
    }
    //毎年の干支をリスト
    static void test4() {
        GregorianCalendar cal = new GregorianCalendar(1700,12-1,25,0,0,0);
        //EasternEphemeris ee = new EasternEphemeris();
        Kansi kansi = new Kansi();
        for(int i=0; i<250; i++) {
            String date = getDate(cal);
            KansiKyuuseiAlmanac.getValue(cal,kansi);
            //ee.getKansi(cal,kansi);
            String y = kansi.get干支(Kansi.YEAR);
            String m = kansi.get干支(Kansi.MONTH);
            String k = kansi.get干支(Kansi.DAY);
            System.out.println(date + y + "," + m + "," + k);
            cal.add(Calendar.YEAR,1);
        }
        
    }
    //毎年の年家九星をリスト
    static void test5() {
        GregorianCalendar cal = new GregorianCalendar(1583,2-1,10,0,0,0);
        //EasternEphemeris ee = new EasternEphemeris();
        Kansi kansi = new Kansi();
        for(int i=0; i<487; i++) {
            String date = getDate(cal);
            KansiKyuuseiAlmanac.getValue(cal,kansi);
            //ee.getKansi(cal,kansi);
            String k = kansi.get干支(Kansi.YEAR);
            String y = kansi.get九星略称(Kansi.YEAR);
            System.out.printf("%s %s %s\n",date,k,y);
            cal.add(Calendar.YEAR,1);
        }        
    }
    //指定年から毎日の日家九星をリスト
    static void test6() {
        GregorianCalendar cal = new GregorianCalendar(1583,1-1,1,0,0,0);
        //EasternEphemeris ee = new EasternEphemeris();
        Kansi kansi = new Kansi();
        for(int i=0; i<60; i++) {
            String date = getDate(cal);
            //ee.getKansi(cal,kansi);
            KansiKyuuseiAlmanac.getValue(cal,kansi);
            String k = kansi.get干支(Kansi.DAY);
            String y = kansi.get九星略称(Kansi.DAY);
            System.out.printf("%s %s %s\n",date,k,y);
            cal.add(Calendar.DAY_OF_MONTH,1);
        }        
    }
    
    static void test3() {
        GregorianCalendar cal = new GregorianCalendar(1873,1-1,1,0,0,0);
        //EasternEphemeris ee = new EasternEphemeris();
        for(int i=0; i<31; i++) {
            String date = getDate(cal);
            Kansi kansi = new Kansi();
            KansiKyuuseiAlmanac.getValue(cal,kansi);
            //ee.getKansi(cal,kansi);
            System.out.println(kansi);
//            String y = kansi.get干支(Kansi.YEAR);
//            String m = kansi.get干支(Kansi.MONTH);
//            String k = kansi.get干支(Kansi.DAY);
//            System.out.println(date + y + "," + m + "," + k);
            cal.add(Calendar.DAY_OF_MONTH,1);
        }        
    }
    public static void main(String [] args) {
        test6();
    }
}
