/*
 * KansiConst.java
 *
 * Created on 2008/01/17, 18:39
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.kyuureki;

/**
 * 干支や九星の名前テーブル。
 * @author 大澤義鷹
 */
public class KansiConst {    
    /**  KansiConst オブジェクトを作成する */
    private KansiConst() {
    }
    /**
     * 十干の名前。
     * "甲","乙","丙","丁","戊","己","庚","辛","壬","癸"
     */
    public static final String [] 十干 = 
        { "甲","乙","丙","丁","戊","己","庚","辛","壬","癸" };
    /**
     * 十二支の名前。
     *  "子","丑","寅","卯","辰","巳","午","未","申","酉","戌","亥"
     */
    public static final String [] 十二支 =
        { "子","丑","寅","卯","辰","巳","午","未","申","酉","戌","亥" };
    /**
     * 九星の名前。
     * null,"一白水星","二黒土星","三碧木星","四緑木星",
     * "五黄土星","六白金星","七赤金星","八白土星","九紫火星"
     */
    public static final String [] 九星 = 
        { null,"一白水星","二黒土星","三碧木星","四緑木星","五黄土星",
          "六白金星","七赤金星","八白土星","九紫火星" };
    /**
     * 九星の略称名。
     * null,"一白","二黒","三碧","四緑",
     * "五黄","六白","七赤","八白","九紫"
     */
    public static final String [] 九星略称 = 
        { null,"一白","二黒","三碧","四緑","五黄",
          "六白","七赤","八白","九紫" };
//    /**
//     * 六十干支のテーブル。
//     * 六十干支[干][支]と代入すれば、六十干支名が求まる。
//     */
//    public static final String [][] 六十干支1 = {
//        { "甲子",null,"甲寅",null,"甲辰",null,"甲午",null,"甲申",null,"甲戌",null },
//        { null,"乙丑",null,"乙卯",null,"乙巳",null,"乙未",null,"乙酉",null,"乙亥" },
//        { "丙子",null,"丙寅",null,"丙辰",null,"丙午",null,"丙申",null,"丙戌",null },
//        { null,"丁丑",null,"丁卯",null,"丁巳",null,"丁未",null,"丁酉",null,"丁亥" },
//        { "戊子",null,"戊寅",null,"戊辰",null,"戊午",null,"戊申",null,"戊戌",null },
//        { null,"己丑",null,"己卯",null,"己巳",null,"己未",null,"己酉",null,"己亥" },
//        { "庚子",null,"庚寅",null,"庚辰",null,"庚午",null,"庚申",null,"庚戌",null },
//        { null,"辛丑",null,"辛卯",null,"辛巳",null,"辛未",null,"辛酉",null,"辛亥" },
//        { "壬子",null,"壬寅",null,"壬辰",null,"壬午",null,"壬申",null,"壬戌",null },
//        { null,"癸丑",null,"癸卯",null,"癸巳",null,"癸未",null,"癸酉",null,"癸亥" }
//    };
    /**
     * 六十干支のテーブル。
     * 六十干支[0〜59]までに順番にならんでいる。
     */
    public static final String [] 六十干支 = {    
        "甲子","乙丑","丙寅","丁卯","戊辰","己巳","庚午","辛未","壬申","癸酉",
        "甲戌","乙亥","丙子","丁丑","戊寅","己卯","庚辰","辛巳","壬午","癸未",
        "甲申","乙酉","丙戌","丁亥","戊子","己丑","庚寅","辛卯","壬辰","癸巳",
        "甲午","乙未","丙申","丁酉","戊戌","己亥","庚子","辛丑","壬寅","癸卯",
        "甲辰","乙巳","丙午","丁未","戊申","己酉","庚戌","辛亥","壬子","癸丑",
        "甲寅","乙卯","丙辰","丁巳","戊午","己未","庚申","辛酉","壬戌","癸亥",
    };
    /**
     * 六十干支[干][支]と代入すれば、六十干支番号が求まる。
     */
    public static final int [][] 六十干支番号表 = {
        { 0, -1, 50, -1, 40, -1, 30, -1, 20, -1, 10, -1 },
        {-1, 1, -1, 51, -1, 41, -1, 31, -1, 21, -1, 11 }, 
        {12, -1, 2, -1, 52, -1, 42, -1, 32, -1, 22, -1 },
        {-1, 13, -1, 3, -1, 53, -1, 43, -1, 33, -1, 23 }, 
        { 24, -1, 14, -1, 4, -1, 54, -1, 44, -1, 34, -1 }, 
        { -1, 25, -1, 15, -1, 5, -1, 55, -1, 45, -1, 35 }, 
        { 36, -1, 26, -1, 16, -1, 6, -1, 56, -1, 46, -1 },
        { -1, 37, -1, 27, -1, 17, -1, 7, -1, 57, -1, 47 }, 
        { 48, -1, 38, -1, 28, -1, 18, -1, 8, -1, 58, -1 }, 
        { -1, 49, -1, 39, -1, 29, -1, 19, -1, 9, -1, 59 }
    };
}
