/*
 * KyuurekiDate.java
 *
 * Created on 2008/01/26, 7:54
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.kyuureki;

/**
 * 旧暦の年月日を表現する。変換結果返却用クラス。<br>
 * KyuurekiAlmanacがグレゴリオ暦から旧暦の日付を返すときは、このオブジェクトに値を
 * 入れて返す。
 * 年閏月日の各値は、フィールド変数から直接行う。get/setメソッドは用意してない。
 * @author 大澤義鷹
 */
public class KyuurekiDate {
    /**
     * 旧暦の月(1-12)
     */
    public int month;
    /**
     * 旧暦の日(1-29または1-30)
     */
    public int day;
    /**
     * 旧暦の年
     */
    public int year;
    /**
     * 閏月のときはtrue、ちがうときはfalse
     */
    public boolean leap;
    
    /**
     * 年、閏フラグ,月,日からオブジェクトを作成する。
     */
    public KyuurekiDate(int year,boolean leap,int month,int day) {
        this.day = day;
        this.month = month;
        this.leap = leap;
        this.year = year;
    }
    /**
     * 閏フラグ,月,日からオブジェクトを作成する。
     */
    public KyuurekiDate(boolean leap,int month,int day) {
        this( 0, leap, month, day );
    }
    /**
     * 空のオブジェクトを作成する
     */
    public KyuurekiDate() {
        
    }
    /**
     * このオブジェクトと指定されたオブジェクトが等しい時にtrueを返す。<br>
     * 等しいときとは、
     * <ul>
     * <li>(1)オブジェクトの型がKyuurekiDateで、かつインスタンスの参照アドレスが同じとき。
     * <li>(2)オブジェクトの型がKyuurekiDateで年、閏、月、日すべてが同じ値のとき。
     * </ul>
     */
    public boolean equals(Object o) {
        if ( o instanceof KyuurekiDate) {
            if( this == o ) return true;
            KyuurekiDate k = (KyuurekiDate)o;
            return month == k.month && day == k.day && 
                     year == k.year  && leap == k.leap;
        }
        return false;
    }
    /**
     * 年,閏,月,日をこのオブジェクトにセットする。
     */
    public void set(int year,boolean leap,int month,int day ) {
        this.year = year;
        this.leap = leap;
        this.month = month;
        this.day = day;
    }
    /**
     * このオブジェクトの文字列表現を返す。<br>
     * 例１ → "旧暦1964年 08月25日"<br>
     * 例２ → "旧暦1987年 閏06月04日"<br>
     */
    public String toString() {
        String ls = ( leap ) ? "閏" : "";
        return String.format("旧暦%04d年 %s%02d月%02d日",year,ls,month,day);
    }
}
