/*
 * KyuurekiLonTest.java
 * JUnit based test
 *
 * Created on 2008/02/21, 18:12
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.kyuureki;

import junit.framework.*;
import java.util.Calendar;

/**
 * KyuurekiAlmanacで、観測地経度による時差補正が正しく計算されるかテストする。
 * @author 大澤義鷹
 */
public class KyuurekiLonTest extends TestCase {
    KyuurekiAlmanac kyuureki;

    public KyuurekiLonTest(String testName) {
        super(testName);
    }

    protected void setUp() throws Exception {
        kyuureki = new KyuurekiAlmanac();
    }

    protected void tearDown() throws Exception {
    }

    String result( Calendar cal, double lon) {
        KyuurekiSupplement sap = new KyuurekiSupplement();
        String kdate = kyuureki.getKyuurekiDate(cal,lon,sap).toString();
        String str =  kdate + "," + sap.toString();
        System.out.println(str);
        return str;
    }
    /**
     * 場合によっては補正日時の秒の桁が、±1秒程度変動してエラーが出る可能性が
     * ある。原因がよくわからない。
     * resultは指定日時と観測地経度から、旧暦日と補正情報を文字列で返し、
     * 正解の文字列と比較している。resultの値が一致しないときはエラーになる。
     */
    public void testKyuureki() {
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.MILLISECOND,0);
        cal.set(1964,9-1,30,22,35,0);
        System.out.println("小笠原 (予測時差) +25分24秒");
        assertEquals( result( cal, 142.1873 ),
            "旧暦1964年 08月26日,日の繰上 true,経度時間補正 28分44秒," +
            "補正日時 1964-10-01(木)23:03:44,夏時間実施 false," +
            "夏時間補正 00分00秒");
        System.out.println("北海道 (予測時差) +25分24秒");
        assertEquals( result( cal, 141.3352 ),
            "旧暦1964年 08月26日,日の繰上 true,経度時間補正 25分20秒," +
            "補正日時 1964-10-01(木)23:00:20,夏時間実施 false," +
            "夏時間補正 00分00秒" );
        System.out.println("沖縄 (予測時差) -29分20秒");
        assertEquals( result( cal, 127.6992 ),
            "旧暦1964年 08月25日,日の繰上 false,経度時間補正 -29分12秒," +
            "補正日時 1964-09-30(水)22:05:47,夏時間実施 false," +
            "夏時間補正 00分00秒" );
        System.out.println("東京 (予測時差) +19分00秒");
        assertEquals( result( cal, 139.7604 ),
            "旧暦1964年 08月25日,日の繰上 false,経度時間補正 19分02秒," +
            "補正日時 1964-09-30(水)22:54:02,夏時間実施 false," +
            "夏時間補正 00分00秒" );
        System.out.println("大阪 (予測時差) +1分56秒");
        assertEquals( result( cal, 135.5287 ),
            "旧暦1964年 08月25日,日の繰上 false,経度時間補正 02分06秒," +
            "補正日時 1964-09-30(水)22:37:06,夏時間実施 false," +
            "夏時間補正 00分00秒" );

    }

}
