/*
 * KyuurekiTest4.java
 *
 * Created on 2008/02/21, 20:02
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.kyuureki;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import junit.framework.TestCase;
import to.tetramorph.time.IntegerDate;

/**
 * 旧暦とグレゴリオ暦の相互変換を行う暦ファイルと、KyuurekiAlmanacによる変換を
 * 比較して、まったく同じ値になるか、すべての日に対して検査を行う。
 * データファイルは、c:/src/perl/kyuureki2/の中にある、
 * "kyuureki1850.csv","kyuureki1920.csv","kyuureki2032.csv"である。
 * @author 大澤義鷹
 */
public class KyuurekiTest4 extends TestCase {
    KyuurekiAlmanac kyuureki;
    static final int DATE = 0;
    static final int NEWMOON = 5;
    static final int MONTH = 6;
    static final int DAY = 7;
    static final int LEAP = 8;
    List<KyuurekiDay> kyuurekiList = new ArrayList<KyuurekiDay>();
    
    /**  KyuurekiTest4 オブジェクトを作成する */
    public KyuurekiTest4(String testName ) {
        super(testName);
    }
    /**
     * 旧暦データファイルをフィールド変数kyuurekiListに読みこむ。
     */
    void load() throws IOException {
        String [] files = { 
            "kyuureki1850.csv","kyuureki1920.csv","kyuureki2032.csv" };
        for(String name : files) {
            File file = new File( "c:/src/perl/kyuureki2/" + name );
            BufferedReader reader = new BufferedReader( new FileReader(file) );
            String [] cols = new String[10];
            int [] dr = new int[3];
            while ( reader.ready() ) {
                String line = reader.readLine();
                String [] temp = line.split(",");
                for(int i=0; i<cols.length; i++) cols[i] = "";
                for(int i=0; i<temp.length; i++) cols[i] = temp[i];
                int ymd = Integer.parseInt( cols[ DATE ] );
                IntegerDate.reverse( ymd,dr );
                int kmonth = Integer.parseInt( cols[MONTH] );
                int kday   = Integer.parseInt( cols[ DAY ] );
                boolean leap = cols[ LEAP ].equals("leap");
                kyuurekiList.add(new KyuurekiDay(ymd,dr[0],leap,kmonth,kday));
                if ( ymd >= kyuureki.getMaximumRange() ) break;
            }
            reader.close();
        }
        // 旧暦年をナンバリングする
        KyuurekiDay date = kyuurekiList.get(0);
        int month = date.month;
        int year = date.year;
        for ( int i=0; i<kyuurekiList.size(); i++ ) {
            KyuurekiDate node = kyuurekiList.get(i);
            if ( node.month == 1 && month == 12) {
                year++;
            }
            node.year = year;
            month = node.month;
        }
    }

    protected void setUp() throws Exception {
        kyuureki = new KyuurekiAlmanac();
    }
    protected void tearDown() throws Exception {
    }
    /**
     * テストメソッド本体。リストにロードされた旧暦データ通りに、KyuurekiAlmanac
     * がグレゴリオ暦を変換するかテストする。
     * @param testname テストの名前で、標準出力にprintlnされる。動作には関係ない。
     */
    public void check( String testname ) throws Exception {
        System.out.println( testname );
        int [] ymd = new int[3];
        long t = System.currentTimeMillis();
        for ( int i=0; i<kyuurekiList.size(); i++ ) {
            KyuurekiDay k = kyuurekiList.get(i);
            IntegerDate.reverse( k.gdate, ymd );
            KyuurekiDate kd = kyuureki.getKyuurekiDate(ymd[0],ymd[1],ymd[2]);
            assertEquals( kd, k);
//            if (! kd.equals( k ) ) {
//                System.out.println(k.gdate + " " + k + " なのに " + kd);
//            }
        }
        System.out.println( "所要時間 " + 
                            (System.currentTimeMillis() - t)/1000d + "[sec]");
        System.out.println( "処理件数 " + kyuurekiList.size() + "件");
        
    }
    
    public void testNormal() throws Exception {
        load();
        check("昇り順アクセステスト");
    }
    
    public void testRandom() throws Exception {
        load();
        Collections.shuffle( kyuurekiList );
        check("ランダムアクセステスト");
    }
    
    public void testReverse() throws Exception {
        load();
        Collections.reverse( kyuurekiList );
        check("降り順アクセステスト");
    }
}
