/*
 * TransitNameTable.java
 *
 * Created on 2008/01/15, 18:25
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.starbase.chartlib;

import java.awt.Color;
import java.util.Calendar;
import to.tetramorph.starbase.chartparts.GBoxLayout;
import to.tetramorph.starbase.chartparts.GLabel;
import to.tetramorph.starbase.lib.Transit;
import to.tetramorph.starbase.util.AngleConverter;

/**
 * ZodiacPanelにトランジットの名前や観測地を表示する。
 * @author 大澤義鷹
 */
public class TransitNameTable extends GBoxLayout {
    private static final int G_CENTER = GLabel.CENTER;
    private static final int G_RIGHT  = GLabel.RIGHT;
    private static final int G_BOTTOM = GLabel.BOTTOM;
    private static final int G_LEFT = GLabel.LEFT;
    private static final int G_TOP = GLabel.TOP;
    //private ZodiacPanel zodiacPanel;
    private GLabel tNameLabel = new GLabel();
    private GLabel tLocationLabel = new GLabel( G_RIGHT, 11 );
    private GLabel tDateLabel = new GLabel( G_RIGHT );
    private GLabel tTimeZoneLabel = new GLabel( G_RIGHT, 11 );
    private GLabel tPlaceLabel = new GLabel( G_RIGHT );
    //private GBoxLayout layout;
    /**
     * TransitNameTable オブジェクトを作成する
     */
    public TransitNameTable() {
        //this.zodiacPanel = zodiacPanel;
        super( GBoxLayout.VERTICAL );
        createTransitLayout();
    }

    /**
     * トランジットの日時、観測地を表示するためのレイアウトを作成
     */
    private void createTransitLayout() {
        tNameLabel.setInset( G_BOTTOM, 0.1 );
        tDateLabel.setInsets( 0, 2, 0.1, 0);
        tTimeZoneLabel.setInset( G_LEFT, 5 );
        tPlaceLabel.setInsets( 0, 5, 0.1,0);
        tLocationLabel.setInset( G_LEFT, 5 );
        //layout = new GBoxLayout( GBoxLayout.VERTICAL );
        add( tNameLabel, tDateLabel, tTimeZoneLabel, tPlaceLabel, tLocationLabel );
        setLocation( 100, 0, G_RIGHT, G_TOP );
//        layout = new GBoxLayout( GBoxLayout.VERTICAL );
//        layout.add( tNameLabel,tDateLabel,tTimeZoneLabel,tPlaceLabel,tLocationLabel);
//        layout.setLocation(100,0,G_RIGHT,G_TOP);
//        zodiacPanel.addGLayout(layout);
    }

//    /**
//     * このオブジェクトをZodiacPanelから削除する。
//     */
//    public void remove() {
//        zodiacPanel.removeGLayout( layout );
//    }
//
//    /**
//     * このオブジェクトを表示する。
//     */
//    public void show() {
//        if ( zodiacPanel.contain( layout ) ) return;
//        zodiacPanel.addGLayout( layout );
//    }

    /**
     * トランジットの日時、観測地等のパラメタをトランジット用レイアウトにセット
     */
    public void setTransit( Transit transit ) {
        String tName = transit.getName();
        if ( tName.length() == 0 ) tName = "トランジット";
        tNameLabel.setText( tName );
        String lat = AngleConverter
                     .getFormattedLatitude( transit.getLatitude() );
        String lon = AngleConverter
                     .getFormattedLongitude( transit.getLongitude() );
        String loc = ( transit.getLatitude() == null ) ?
                 "〔観測点不明〕"     :     lat + ", " + lon;
        tLocationLabel.setText( loc );
        int dst_offset = transit.getCalendar().get(Calendar.DST_OFFSET);
        String displayName = transit.getTimeZone().getDisplayName();
        if ( dst_offset > 0 ) {
            displayName = displayName.concat("〔夏時間実施中〕");
        }
        tTimeZoneLabel.setText( displayName );
        String date = transit.getFormattedDate();
        if ( transit.getTime() == null ) date += "〔時刻不明〕";
        tDateLabel.setText( date );
        String tPName = transit.getPlaceName().length() == 0 ?
            "〔地名未登録〕"     :     transit.getPlaceName();
        tPlaceLabel.setText( tPName );
    }

    public void setColors( Color nameColor,
                             Color dateColor,
                             Color placeColor ) {
        tNameLabel.setColor( nameColor );
        tDateLabel.setColor( dateColor );
        tLocationLabel.setColor( placeColor );
        tPlaceLabel.setColor( placeColor );
        tTimeZoneLabel.setColor( placeColor );
    }
}
