/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.starbase.dict;

import java.awt.Component;
import java.io.File;
import javax.swing.JFileChooser;
import to.tetramorph.util.AmateruBookFileFilter;
import to.tetramorph.util.XMLFileFilter;

/**
 * ブックファイルを選択する際のファイルチューザー。
 * カレントディレクトリを記憶する機能つき。
 * 拡張子がABFとXML用のファイルフィルタを設定している。
 * @author 大澤義鷹
 */
class ImportBookDialog {
    private static final JFileChooser chooser;

    static {
        chooser = new JFileChooser();
        //すべてを透過するデフォルトのフィルタを禁止にする
        chooser.setAcceptAllFileFilterUsed(false);
        //専用のフィルタを設定
        chooser.addChoosableFileFilter(new XMLFileFilter());
        chooser.addChoosableFileFilter(new AmateruBookFileFilter());
        chooser.setDialogTitle("ブックファイルの選択");
    }

    /**
     * ブックファイル選択用のファイルチューザを開く。
     * @param owner 親となるコンポーネント
     * @return 選択されたファイル。未選択のときはnullを返す。
     */
    public static File show(Component owner) {
        //System.out.println("EDT = " + SwingUtilities.isEventDispatchThread());
        String path = Config.usr.getProperty("BookChooser.path", "");
        if ( ! path.isEmpty() ) {
            chooser.setCurrentDirectory(new File(path));
        }
        int result = chooser.showOpenDialog(owner);
        if (result != JFileChooser.APPROVE_OPTION) return null; //選択が中止された
        File  file = chooser.getSelectedFile();
        Config.usr.setProperty("BookChooser.path",
                 chooser.getCurrentDirectory().getAbsolutePath());
        return file;
    }
}
