/*
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */
package to.tetramorph.starbase.formatter;
import javax.swing.text.PlainDocument;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;

/**
 * テキストフィールドの入力文字数に制限をかけるためのもの。
 * <pre>
 * textfield.setDocument( new LimitedDocument( 10 ) );
 * などとして使う。
 * </pre>
 * @see javax.swing.JTextField
 */
public class LimitedDocument extends PlainDocument {
  
  int limit;
  /**
   * 指定された文字数でオブジェクトを作成する。これをセットされたJTextFieldは、
   * 入力文字数に制限がかかる。
   * @see javax.swing.JTextField
   */
  public LimitedDocument( int limit ){
    this.limit = limit;
  }
  /**
   * プログラマがこのメソッドを呼び出す必要は無い。
   */
  public void insertString( int offset, String str, AttributeSet a ) {   
    if( getLength() >= limit ) return;
    try {
      super.insertString( offset, str, a );
    } catch( BadLocationException e ) {
      System.out.println( e );
    }
  }
}
