/*
 * GuestDatabase.java
 *
 * Created on 2007/03/11, 9:36
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.starbase.util;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import javax.swing.tree.TreePath;

/**
 * チャートモジュールやサーチモジュール側でデータベースと接続する際には、
 * このクラスを使用する。シングルトンクラスでインスタンスはつねに一つ。
 * DBのと接続はシステム終了時に自動的にクローズされる。
 * あらかじめHSQLDBに登録されたguestユーザでDBにログインする。guestアカウントは
 * DBに対してSELECTする権限のみもち、INSERT、DELETE、CREATEなどは禁止されている。
 * 天体位置を検索するときにつかわれるストアドプロシージャは使用可能。
 * @author 大澤義鷹
 */
public class GuestDatabase {
  private Connection con;
  private static GuestDatabase db = new GuestDatabase();
  //  GuestDatabase オブジェクトを作成する
  private GuestDatabase() {
    try {
      Class.forName("org.hsqldb.jdbcDriver");
    } catch ( ClassNotFoundException e ) {
      e.printStackTrace();
    }
    try {
      String driverURL = "jdbc:hsqldb:hsql://localhost";
      con = DriverManager.getConnection(driverURL,"guest","");
      System.out.println("データベース接続完了(guest account)");
    } catch (SQLException e) {
      e.printStackTrace();
      throw new IllegalStateException(e);
    }
    //シャットダウンのときに呼び出されるスレッドを登録
    Runtime.getRuntime().addShutdownHook(new Thread() {
      public void run() {
        try {
          if(con != null) {
            con.close();
            System.out.println("DBとの接続をクローズ(Guest account)");
          }
        } catch( Exception e ) { e.printStackTrace(); }
      }
    });
  }
  /**
   * インスタンスを返す。
   */
  public static GuestDatabase getInstance() {
    return db;
  }
  /**
   * データベースとのConnectionオブジェクトを返す。
   */
  public Connection getConnection() {
    return con;
  }
}
