/*
 * AngleFractionSpinner.java
 *
 * Created on 2008/11/21, 21:12
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.starbase.widget;

import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import to.tetramorph.starbase.lib.Unit;
import to.tetramorph.starbase.util.AngleConverter;

/**
 * 十進モード、六十進モードに応じて、0〜59または0から99までの値を入力可能な
 * スピナー。モードはAngleConverter.getAngleUnit()から取得している。
 * @author 大澤義鷹
 */
public class AngleFractionSpinner extends JSpinner {
    private int max;

    /**  
     * AngleFractionSpinner オブジェクトを作成する 
     */
    public AngleFractionSpinner() {
        super();
        updateUnit();
    }
    
    /**
     * このスピナーの単位系を変更する。現在設定されている単位系プロパティの値に
     * したがって、このスピナーのモデルを作り替えてセットする。
     */
    public void updateUnit() {
        max = ( AngleConverter.getAngleUnit() == AngleConverter.DECIMAL ) ?
            99 : 59;
        SpinnerNumberModel model = new SpinnerNumberModel(0, 0, max, 1);
        this.setModel( model );        
    }
    
    /**
     * このスピナーの値を十進の浮動小数にして返す。六十進モードのときは、十進に
     * 変換した値を返す。値は0.99とか0以下の値。
     */
    public double getFraction() {
        double v = (double)((Integer)getValue());
        if ( max == 99 )
            return v / 100.;
        return Unit.decimal( v / 100. );
    }
}
