/*
 * SingleSideBorder.java
 *
 * Created on 2007/11/12, 1:41
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */
package to.tetramorph.starbase.widget;

import java.awt.Component;
import java.awt.Graphics;
import javax.swing.border.EtchedBorder;

/**
 * ボーダーは通常矩形を囲むように描画されるが、これは矩形の上の線だけを描画する。
 * @author 大澤義鷹
 */
public class SingleSideBorder extends EtchedBorder {

    /**
     * 矩形の上方にエッチング線を引く。
     */
    public static final int TOP = 0;
    /**
     * 矩形の左方にエッチング線を引く。
     */
    public static final int LEFT = 1;
    int orientation;

    /**
     * SingleSideBorder オブジェクトを作成する
     * @param orientation LEFT,TOPのいずれか。
     */
    public SingleSideBorder(int orientation) {
        super();
        this.orientation = orientation;
    }

    @Override
    public void paintBorder(Component c, Graphics g, int x, int y, int width, int height) {
        int w = width;
        int h = height;

        g.translate(x, y);
        if (orientation == TOP) {
            g.setColor(etchType == LOWERED ? getShadowColor(c) : getHighlightColor(c));
            g.drawLine(0, 0, w - 1, 0);
            g.setColor(etchType == LOWERED ? getHighlightColor(c) : getShadowColor(c));
            g.drawLine(0, 1, w - 1, 1);
        } else if (orientation == LEFT) {
            g.setColor(etchType == LOWERED ? getShadowColor(c) : getHighlightColor(c));
            g.drawLine(0, 0, 0, h - 1);
            g.setColor(etchType == LOWERED ? getHighlightColor(c) : getShadowColor(c));
            g.drawLine(1, 0, 1, h - 1);
        }
        g.translate(-x, -y);
    }
}
