/*
 * WordBalloonHandler.java
 *
 * Created on 2006/11/14, 10:12
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.starbase.widget;

/**
 * ワードバルーンを表示するときは、オンカーソル中であることが条件になるが、
 * それを検査するメソッドを定義しているインターフェイス。
 *
 * ホロスコープ上の天体にオンカーソルしてコンマ数秒おいてからワードバルーンが出て
 * キャプションを表示させたい(ToolTipsと同じ動作)。
 * オンカーソルしたときに、WordBalloonのshowメソッドを呼び出すが、その後時間を
 * おかずにアウトカーソルしてしまった場合、ワードバルーンを出さないようにしたい。
 * このインターフェイスのgetSelectedObject()は、オンカーソルしたオブジェクトを
 * 返す。アウトカーソルしてしまったときはnullを返す。
 * showメソッド呼びだしのとき、showメソッド内では呼び出された時点でのオンカーソル
 * オブジェクトの参照アドレスをキープする。タイマーが起動され一定時間後、
 * もう一度WrodBalloonHandler#getSelectedObject()が呼び出され、showメソッド呼びだし
 * のときと同じオブジェクトであれば、一定時間アウトカーソルもしなかったし、一定
 * 時間の間に隣の別のオブジェクトに選択が映ったわけでもないとみなし、ワードバルーン
 * を出すという処理をする。
 * もちろん一定時間内に別の天体にカーソルが行って一定時間内に戻ってきた場合は、
 * アウトカーソルしなかったとみなされる。
 */
public interface WordBalloonHandler {
  /**
   * 現在選択中のオブジェクトを返す。
   */
  public Object getSelectedObject();
  /**
   * 選択中のオブジェクトをセットする。WordBalloonはワードバルーン消去後、
   * このメソッドをつかって選択中オブジェクトにnullをセットする。
   */
  public void setSelectedObject(Object o);
}
