/*
 * ImageFileFilter.java
 *
 * Created on 2008/10/23, 8:18
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.util;

import java.io.File;
import javax.swing.filechooser.FileFilter;

/**
 * JFileChooser用のファイルフィルタで、jpgやpngなどの拡張子をもったものだけ透過
 * する。
 * @author 大澤義孝
 */
public class ImageFileFilter extends FileFilter {
    /**
     * コンストラクタに指定するファイル形式名 値は"tif" 
     */
    public static final String TIFF = "tif";
    /**
     * コンストラクタに指定するファイル形式名 値は"gif" 
     */
    public static final String GIF  = "gif";
    /**
     * コンストラクタに指定するファイル形式名 値は"jpg" 
     */
    public static final String JPEG = "jpg";
    /**
     * コンストラクタに指定するファイル形式名 値は"png" 
     */
    public static final String PNG =  "png";
    /**
     * コンストラクタに指定するファイル形式名 値は"bmp" 
     */
    public static final String BMP =  "bmp";
    /**
     * コンストラクタに指定するファイル形式名 値は"wbmp" 
     */
    public static final String WBMP =  "wbmp";
    /**
     * フィールドPNG,GIF,BMP,TIFF,JPEG,WBMPを配列化したもの。
     */
    public static final String [] FORMAT_NAMES = {
        PNG,GIF,BMP,TIFF,JPEG,WBMP
    };
    private String format;
    /**
     * このファイルフィルタが透過する拡張子の名前を、
     * ImageFileFilter.TIFF,GIF,JPEG,PNG,BMP,WBMPの中から指定する。
     */
    public ImageFileFilter( String format ) {
        this.format = format;
    }
    
    /**
     * 入力されたファイルfがディレクトリの場合と、ファイルの拡張子が、
     * コンストラクタで指定されたtypeと同じものの場合はtrueを返す。
     * 拡張子の大文字小文字は区別しない。なおjpgはjpegでも同じと認識する。
     * tifはtiffでも同じと認識する。
     */
    public boolean accept( File f ) {
        if ( f.isDirectory() ) {
            return true;
        }
        String ext= FileFilterUtils.getExtension(f);
        if ( ext != null ) {
            if ( format.equals( JPEG ) &&
                 ( ext.equals( JPEG ) || ext.equals("jpeg") ) )
                return true;
            if ( format.equals( TIFF ) && 
                 ( ext.equals( TIFF ) || ext.equals("tiff") ) )
                return true;
            if ( format.equals( ext ) )
                return true;
        }
        return false;
    }
    

    /**
     * このフィルダーが透過する拡張子名を返す。
     */
    public String getDescription() {
        return format.toUpperCase() + " イメージ";
    }
    
    /**
     * コンストラクタで渡されたタイプ名を返す。
     */
    public String getFormatName() {
        return format;
    }
}
