/*
 * HousePreferencePanel.java
 *
 * Created on 2006/09/15, 6:14
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.starbase;

import to.tetramorph.starbase.lib.Const;
import to.tetramorph.util.Preference;

/**
 * デフォルトのハウス分割法や、カスプが計算できない場合のハウス分割法を設定
 * するためのパネル。MainConfigDialogに集約される。
 *<pre>
 * CuspUnknownHouseSystem = 1 or 2
 * 　　1ならソーラー、2ならソーラーサイン
 * RegistPlace = true/false
 *     trueなら「ただし時間が未登録だが場所は指定されているときは、
 * 　　デフォルトタイムで計算しデフォルトのハウス分割法を使用
 * HouseSystemIndex = 0 .. 7
 * 　　ハウス分割法コンボボックスの選択中インデックス番号。
 * DefaultHouseSystem =  1文字
 * 　"P","K","O","R","C","E","1","2"のいずれか。1はソーラー2はソーラーハウス。
 *   その他はSwissEphのハウスシステム指定パラメターに従う。
 * PrioritizeSolar = true/false
 *   時刻や緯度・経度が登録されていないときソーラーハウス(1)か、ソーラーサイン
 * を使用する。
 *</pre>
 * HouseSystemIndexとDefaultHouseSystemは対になっていて、index番号が指している
 * 一文字と、コンボボックスに入っている分割法名は対応している。
 * @author  大澤義鷹
 */
class HousePreferencePanel extends PreferencePanel {
    //ButtonGroup cuspButtonGroup2 = new ButtonGroup();
    /** オブジェクトを作成する。 */
    public HousePreferencePanel() {
        initComponents();
        init();
    }
    // ハウス設定の初期化個別
    private void init() {
        solarButtonGroup.add( solarRadioButton1 );
        solarButtonGroup.add( solarRadioButton2 );
        cuspButtonGroup.add( unknownCuspRadioButton1 );
        cuspButtonGroup.add( unknownCuspRadioButton2 );
        int select = Integer.parseInt(
            Config.usr.getProperty("CuspUnknownHouseSystem","1") );
        if ( select == 1 )
            unknownCuspRadioButton1.doClick();
        else
            unknownCuspRadioButton2.doClick();
        if ( Config.usr.getBoolean("PrioritizeSolar", true ) )
            solarRadioButton1.doClick();
        else
            solarRadioButton2.doClick();
        houseSystemComboBox.removeAllItems();
        for ( String name : Const.HOUSE_SYSTEM_NAMES )
            houseSystemComboBox.addItem( name );
        houseSystemComboBox.setSelectedIndex(
            Config.usr.getInteger("HouseSystemIndex", 0) );
    }
    /**
     * 設定をprefに登録する。
     */
    @Override
    public void regist() {
        String cmd = cuspButtonGroup.getSelection().getActionCommand();
        Config.usr.setProperty("CuspUnknownHouseSystem",cmd);
        Config.usr.setProperty("PrioritizeSolar",""+solarRadioButton1.isSelected());
        int index = houseSystemComboBox.getSelectedIndex();
        Config.usr.setProperty("HouseSystemIndex",""+index);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;
        javax.swing.JLabel jLabel3;
        javax.swing.JLabel jLabel4;
        javax.swing.JLabel jLabel5;

        solarButtonGroup = new javax.swing.ButtonGroup();
        cuspButtonGroup = new javax.swing.ButtonGroup();
        borderPanel = new javax.swing.JPanel();
        jLabel3 = new javax.swing.JLabel();
        unknownCuspRadioButton1 = new javax.swing.JRadioButton();
        unknownCuspRadioButton2 = new javax.swing.JRadioButton();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        houseSystemComboBox = new javax.swing.JComboBox();
        solarRadioButton1 = new javax.swing.JRadioButton();
        solarRadioButton2 = new javax.swing.JRadioButton();

        setLayout(new java.awt.GridLayout(1, 0));

        setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 16, 1, 16));
        borderPanel.setLayout(new java.awt.GridBagLayout());

        borderPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 16, 16, 1));
        jLabel3.setText("\u6642\u9593\u3084\u5834\u6240\u304c\u4e0d\u660e\u3067\u30ab\u30b9\u30d7\u8a08\u7b97\u304c\u3067\u304d\u306a\u3044\u5834\u5408\u306e\u30cf\u30a6\u30b9\u5206\u5272\u65b9\u6cd5");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(3, 0, 3, 0);
        borderPanel.add(jLabel3, gridBagConstraints);

        unknownCuspRadioButton1.setSelected(true);
        unknownCuspRadioButton1.setText("\u592a\u967d\u30b5\u30a4\u30f3\u306e0\u5ea6\u3092\u4e00\u5ba4\u30ab\u30b9\u30d7\u3068\u3057\u3066\u30a4\u30b3\u30fc\u30eb\u5206\u5272 ( \u30bd\u30fc\u30e9\u30fc\u30b5\u30a4\u30f3 )");
        unknownCuspRadioButton1.setActionCommand("1");
        unknownCuspRadioButton1.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        unknownCuspRadioButton1.setMargin(new java.awt.Insets(0, 0, 0, 0));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(3, 20, 3, 0);
        borderPanel.add(unknownCuspRadioButton1, gridBagConstraints);

        unknownCuspRadioButton2.setText("\u592a\u967d\u306e\u4f4d\u7f6e\u3092\u4e00\u5ba4\u30ab\u30b9\u30d7\u3068\u3057\u3066\u30a4\u30b3\u30fc\u30eb\u5206\u5272 ( \u30bd\u30fc\u30e9\u30fc )");
        unknownCuspRadioButton2.setActionCommand("2");
        unknownCuspRadioButton2.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        unknownCuspRadioButton2.setMargin(new java.awt.Insets(0, 0, 0, 0));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(3, 20, 3, 0);
        borderPanel.add(unknownCuspRadioButton2, gridBagConstraints);

        jLabel4.setText("\u3000\u3000");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.insets = new java.awt.Insets(3, 0, 3, 0);
        borderPanel.add(jLabel4, gridBagConstraints);

        jLabel5.setText("\u30c7\u30d5\u30a9\u30eb\u30c8\u306e\u30cf\u30a6\u30b9\u5206\u5272\u6cd5");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(3, 0, 3, 0);
        borderPanel.add(jLabel5, gridBagConstraints);

        houseSystemComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "\u30d7\u30e9\u30b7\u30fc\u30c0\u30b9", "\u30b3\u30c3\u30db", "\u30dd\u30eb\u30d5\u30e5\u30ea\u30aa\u30b9", "\u30ec\u30ae\u30aa\u30e2\u30f3\u30bf\u30ca\u30b9", "\u30ad\u30e3\u30f3\u30d1\u30ca\u30b9", "\u30a4\u30b3\u30fc\u30eb", "\u30bd\u30fc\u30e9\u30fc\u30b5\u30a4\u30f3", "\u30bd\u30fc\u30e9\u30fc" }));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(3, 0, 8, 0);
        borderPanel.add(houseSystemComboBox, gridBagConstraints);

        solarRadioButton1.setSelected(true);
        solarRadioButton1.setText("\u30bd\u30fc\u30e9\u30fc\u307e\u305f\u306f\u30bd\u30fc\u30e9\u30fc\u30b5\u30a4\u30f3\u3092\u4f7f\u7528\u3059\u308b\u3002");
        solarRadioButton1.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        solarRadioButton1.setMargin(new java.awt.Insets(0, 0, 0, 0));
        solarRadioButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                solarRadioButton1ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(3, 0, 3, 0);
        borderPanel.add(solarRadioButton1, gridBagConstraints);

        solarRadioButton2.setText("\u30c7\u30d5\u30a9\u30eb\u30c8\u306e\u5730\u65b9\u6642\u3068\u89b3\u6e2c\u5730\u3068\u30cf\u30a6\u30b9\u5206\u5272\u6cd5\u3067\u8a08\u7b97\u3059\u308b\u3002");
        solarRadioButton2.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        solarRadioButton2.setMargin(new java.awt.Insets(0, 0, 0, 0));
        solarRadioButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                solarRadioButton2ActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(3, 0, 3, 0);
        borderPanel.add(solarRadioButton2, gridBagConstraints);

        add(borderPanel);

    }// </editor-fold>//GEN-END:initComponents

  private void solarRadioButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_solarRadioButton2ActionPerformed
      unknownCuspRadioButton1.setEnabled(false);
      unknownCuspRadioButton2.setEnabled(false);
  }//GEN-LAST:event_solarRadioButton2ActionPerformed

  private void solarRadioButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_solarRadioButton1ActionPerformed
      unknownCuspRadioButton1.setEnabled(true);
      unknownCuspRadioButton2.setEnabled(true);
  }//GEN-LAST:event_solarRadioButton1ActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel borderPanel;
    private javax.swing.ButtonGroup cuspButtonGroup;
    private javax.swing.JComboBox houseSystemComboBox;
    private javax.swing.ButtonGroup solarButtonGroup;
    private javax.swing.JRadioButton solarRadioButton1;
    private javax.swing.JRadioButton solarRadioButton2;
    private javax.swing.JRadioButton unknownCuspRadioButton1;
    private javax.swing.JRadioButton unknownCuspRadioButton2;
    // End of variables declaration//GEN-END:variables

}
