/*
 * MainConfigDialog.java
 *
 * Created on 2006/09/15, 0:49
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.starbase;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import to.tetramorph.util.ParentWindow;

/**
 * デフォルトタイムやノードタイプなどスターベース全体の設定を行うダイアログ。
 * showDialog()したときに与えたConfig.dataに従って設定情報を反映。
 * （Config.dataは不揮発性にしたPreferenceオブジェクト。）
 * 登録ボタンがおされたらそのときの設定情報を書き込み。
 * 与えられるPreferanceには必要なプロパティのキーが全て設定されていること。
 * 入っていないとNullPointerExceptionなどが出る。
 * 中止が押されたときは、なにもしないので値は変化しない。
 */
class MainConfigDialog extends javax.swing.JDialog {

    private static MainConfigDialog dialog = null;
    private PreferencePanel [] prefPanels; // = new PreferencePanel[4];
    private boolean isUpdate = false;

    //シングルトンとする
    private MainConfigDialog( Frame parent ) {
        super(parent,true);
        initComponents();
        ParentWindow.setEscCloseOperation(this,new AbstractAction() {
            @Override
            public void actionPerformed(ActionEvent evt) {
                abortButton.doClick();
            }
        });
        prefPanels = new PreferencePanel[] {
            new TimePreferencePanel(),
            new NodePreferencePanel(),
            new HousePreferencePanel(),
            new TransitPlacePreferencePanel(),
            new UnitPreferencePanel(),
            new WordBalloonPreferencePanel()
        };
        tabbedPane.add("時間",prefPanels[0]);
        tabbedPane.add("ノード",prefPanels[1]);
        tabbedPane.add("ハウス",prefPanels[2]);
        tabbedPane.add("観測地",prefPanels[3]);
        tabbedPane.add("単位系",prefPanels[4]);
        tabbedPane.add("ガイド",prefPanels[5]);
        pack();
    }

    /**
     * 設定ダイアログを開く。登録ボタンが押されたときは設定情報をprefに書きこんで
     * 返す。「中止ボタンが押されたときはなにもせず戻る。
     * @param pref 設定情報が入っているPreferenceオブジェクト。
     * @param owner このダイアログの親となるフレームオブジェクト
     * @return 登録されたときはtrue。中止されたときはfalse。
     */
    public static boolean showDialog(java.awt.Frame owner ) {
        dialog = new MainConfigDialog( owner );
        dialog.setLocationRelativeTo( null );
        dialog.setVisible( true );
        return dialog.isUpdate;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        tabbedPane = new javax.swing.JTabbedPane();
        buttonPanel = new javax.swing.JPanel();
        registButton = new javax.swing.JButton();
        abortButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("環境設定");
        getContentPane().add(tabbedPane, java.awt.BorderLayout.CENTER);

        registButton.setMnemonic('Y');
        registButton.setText("登録(Y)");
        registButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                registButtonActionPerformed(evt);
            }
        });
        buttonPanel.add(registButton);

        abortButton.setMnemonic('N');
        abortButton.setText("中止(N)");
        abortButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                abortButtonActionPerformed(evt);
            }
        });
        buttonPanel.add(abortButton);

        getContentPane().add(buttonPanel, java.awt.BorderLayout.SOUTH);

        pack();
    }// </editor-fold>//GEN-END:initComponents
  //設定中止
  private void abortButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_abortButtonActionPerformed
      isUpdate = false;
      dispose();
  }//GEN-LAST:event_abortButtonActionPerformed
  //設定登録
  private void registButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_registButtonActionPerformed
      for ( int i = 0; i < prefPanels.length; i++ )
          prefPanels[i].regist();
      Config.save();
      isUpdate = true;
      dispose();
      //MainFrame側で必要に応じて天体位置の再計算が行われる。
  }//GEN-LAST:event_registButtonActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton abortButton;
    private javax.swing.JPanel buttonPanel;
    private javax.swing.JButton registButton;
    private javax.swing.JTabbedPane tabbedPane;
    // End of variables declaration//GEN-END:variables

}
