/*
 * UnitPreferencePanel.java
 *
 * Created on 2008/01/06, 12:07
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.starbase;

/**
 * 10進数/60進数どちらの単位系を使用するか設定するパネル。
 * デフォルトは10進数になっている。
 * <pre>
 * 設定されるキー
 *   "app.topounit"  土地の単位系で60進を使うときは"60"、10進数のときは"10"。
 *   "app.angleunit" 天体位置の角度に使う単位系で60進なら"60"、10進なら"10"。
 * </pre>
 * @author  大澤義鷹
 */
class UnitPreferencePanel extends PreferencePanel {

    /**
     * オブジェクトを作成する。
     * @param pref Config.dataを指定する。
     */
    public UnitPreferencePanel() {
        initComponents();
        if ( Config.usr.getProperty("app.topounit","10").equals("60") )
            topoRadioButton2.setSelected(true);
        else
            topoRadioButton1.setSelected(true);
        if ( Config.usr.getProperty("app.angleunit","10").equals("60") )
            bodyRadioButton2.setSelected(true);
        else
            bodyRadioButton1.setSelected(true);
    }
    /**
     * 他のPreferencePanelと少々異なり、登録の際にSystemプロパティに
     * "app.topounit","app.angleunit"のキーで値を書きこむ。
     * またコンストラクタで与えられたprefにも同じ値を書きこむ。
     * このキーを参照するクラスは、通常はSystemプロパティから値を参照する。
     */
    @Override
    public void regist() {
        String v1 = topoButtonGroup.getSelection().getActionCommand();
        Config.usr.setProperty("app.topounit",v1);
        System.setProperty("app.topounit",v1);

        String v2 = bodyButtonGroup.getSelection().getActionCommand();
        Config.usr.setProperty("app.angleunit",v2);
        System.setProperty("app.angleunit",v2);
    }
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;
        javax.swing.JLabel jLabel1;
        javax.swing.JLabel jLabel2;
        javax.swing.JLabel jLabel3;
        javax.swing.JLabel jLabel4;
        javax.swing.JLabel jLabel5;
        javax.swing.JLabel jLabel6;
        javax.swing.JPanel jPanel1;

        topoButtonGroup = new javax.swing.ButtonGroup();
        bodyButtonGroup = new javax.swing.ButtonGroup();
        jPanel1 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        topoRadioButton1 = new javax.swing.JRadioButton();
        topoRadioButton2 = new javax.swing.JRadioButton();
        jLabel2 = new javax.swing.JLabel();
        bodyRadioButton1 = new javax.swing.JRadioButton();
        bodyRadioButton2 = new javax.swing.JRadioButton();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();

        topoButtonGroup.add(topoRadioButton1);
        topoButtonGroup.add(topoRadioButton2);
        bodyButtonGroup.add( bodyRadioButton1 );
        bodyButtonGroup.add( bodyRadioButton2 );

        setLayout(new java.awt.GridLayout());

        jPanel1.setLayout(new java.awt.GridBagLayout());

        jLabel1.setText("\u89b3\u6e2c\u5730\u306e\u7def\u5ea6\u30fb\u7d4c\u5ea6\u306e\u8868\u793a\u3068\u5165\u529b\u3067\u306e\u5358\u4f4d\u7cfb");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 9, 0);
        jPanel1.add(jLabel1, gridBagConstraints);

        topoRadioButton1.setSelected(true);
        topoRadioButton1.setText("10\u9032\u6570");
        topoRadioButton1.setActionCommand("10");
        topoRadioButton1.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        topoRadioButton1.setMargin(new java.awt.Insets(0, 0, 0, 0));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(0, 15, 0, 0);
        jPanel1.add(topoRadioButton1, gridBagConstraints);

        topoRadioButton2.setText("60\u9032\u6570");
        topoRadioButton2.setActionCommand("60");
        topoRadioButton2.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        topoRadioButton2.setMargin(new java.awt.Insets(0, 0, 0, 0));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(4, 15, 0, 0);
        jPanel1.add(topoRadioButton2, gridBagConstraints);

        jLabel2.setText("\u7363\u5e2f\u5ea7\u6a19\u306e\u8868\u793a\u3068\u5165\u529b\u3067\u306e\u5358\u4f4d\u7cfb");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(9, 0, 9, 0);
        jPanel1.add(jLabel2, gridBagConstraints);

        bodyRadioButton1.setSelected(true);
        bodyRadioButton1.setText("10\u9032\u6570");
        bodyRadioButton1.setActionCommand("10");
        bodyRadioButton1.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        bodyRadioButton1.setMargin(new java.awt.Insets(0, 0, 0, 0));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(0, 15, 0, 0);
        jPanel1.add(bodyRadioButton1, gridBagConstraints);

        bodyRadioButton2.setText("60\u9032\u6570");
        bodyRadioButton2.setActionCommand("60");
        bodyRadioButton2.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        bodyRadioButton2.setMargin(new java.awt.Insets(0, 0, 0, 0));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(4, 15, 0, 0);
        jPanel1.add(bodyRadioButton2, gridBagConstraints);

        jLabel3.setText("\"35.95734N\" \u7b49");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 23, 0, 0);
        jPanel1.add(jLabel3, gridBagConstraints);

        jLabel4.setText("\"35  59  59.123S\" \u7b49");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(4, 23, 0, 0);
        jPanel1.add(jLabel4, gridBagConstraints);

        jLabel5.setIcon(new javax.swing.ImageIcon(getClass().getResource("/resources/symbols/lib.gif")));
        jLabel5.setText("16.99    \u7b49");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 23, 0, 0);
        jPanel1.add(jLabel5, gridBagConstraints);

        jLabel6.setIcon(new javax.swing.ImageIcon(getClass().getResource("/resources/symbols/gem.gif")));
        jLabel6.setText("16\u309c59\u2019 \u7b49");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(4, 23, 0, 0);
        jPanel1.add(jLabel6, gridBagConstraints);

        add(jPanel1);

    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup bodyButtonGroup;
    private javax.swing.JRadioButton bodyRadioButton1;
    private javax.swing.JRadioButton bodyRadioButton2;
    private javax.swing.ButtonGroup topoButtonGroup;
    private javax.swing.JRadioButton topoRadioButton1;
    private javax.swing.JRadioButton topoRadioButton2;
    // End of variables declaration//GEN-END:variables

}
