/*
 * PanelBar.java
 *
 * Created on 2007/11/02, 22:09
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.starbase.multisplit;

import java.awt.Color;
import java.awt.Component;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import javax.swing.AbstractButton;
import javax.swing.JRadioButton;
import javax.swing.border.BevelBorder;
import to.tetramorph.starbase.widget.PanelCloseButton;
import to.tetramorph.starbase.widget.PanelComebackButton;
import to.tetramorph.starbase.widget.PanelMinimizeButton;
import to.tetramorph.starbase.widget.PanelVisibleButton;

/**
 * クローズボタンなどがついたタイトルバー。単純にJPanelに部品が取り付けられたもの。
 * シャッターペイン用のハンドルバーなどに使用している。
 * @author  大澤義鷹
 */
public class PanelBar extends javax.swing.JPanel {
  /** 「閉じる」ボタン */
  public static final int CLOSE_BUTTON = 0;
  /** 「最小化」ボタン */
  public static final int MINIMIZE_BUTTON = 1;
  /** 「復帰」ボタン */
  public static final int COMEBACK_BUTTON = 2;
  /** 「表示/非表示」ボタン。これはトグルボタン。 */
  public static final int VISIBLE_BUTTON = 3;
  
  final static Color focusColor = new Color(72,143,251);
  AbstractButton [] buttons = new AbstractButton[] {
    new PanelCloseButton(),
    new PanelMinimizeButton(), 
    new PanelComebackButton(),
    new PanelVisibleButton()
  };
  
  /**
   * Creates new form PanelBar
   */
  public PanelBar() {
    initComponents();
    setBorder(new MyBorder());
  }

  public void setButtons(int ... buttonCodes ) {
    buttonPanel.removeAll();
    for(int i : buttonCodes ) buttonPanel.add( buttons[i] );
    validate();
  }

  public AbstractButton getButton(int buttonCode) {
    return buttons[buttonCode];
  }
  
  public void setTitle(String title) {
    titleLabel.setText(title);
  }
  
  class MyBorder extends BevelBorder {
    MyBorder() {
      super( BevelBorder.RAISED );
    }
    protected void paintRaisedBevel(Component c, Graphics gg, int x, int y,
                                    int width, int height)  {
        Graphics2D g = (Graphics2D)gg;
        Color oldColor = g.getColor();
        int h = height;
        int w = width;
        g.translate(x, y);
        int  x2 = x + w-1; //(int)(w * 0.8);
        int  y2 = y;

        GradientPaint gp = new GradientPaint(x,y,focusColor,x2,y2,getBackground(),false );
        g.setPaint(gp);
        g.fill(new Rectangle(x+1,y+1,w-3,h-1));

        g.setColor(getHighlightOuterColor(c));
        g.drawLine(0, 1, 0, h-1); //左
        g.drawLine(1, 1, w-2, 1); //上

        g.setColor(getShadowOuterColor(c));
        g.drawLine(1, h-1, w-1, h-1); //下
        g.drawLine(w-1, 0, w-1, h-1); //右
        g.drawLine(0,0,w-1,0); //最上線

//        g.setColor(getHighlightInnerColor(c));
//        g.setColor(getShadowOuterColor(c));
//        g.setColor(getShadowInnerColor(c));

        g.translate(-x, -y);
        g.setColor(oldColor);

    }
    
  }
  
  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  // <editor-fold defaultstate="collapsed" desc=" Generated Code">//GEN-BEGIN:initComponents
  private void initComponents() {
    java.awt.GridBagConstraints gridBagConstraints;

    jPanel2 = new javax.swing.JPanel();
    titleLabel = new javax.swing.JLabel();
    jPanel3 = new javax.swing.JPanel();
    buttonPanel = new javax.swing.JPanel();
    panelCloseButton1 = new to.tetramorph.starbase.widget.PanelCloseButton();

    setLayout(new java.awt.BorderLayout());

    jPanel2.setLayout(new java.awt.GridBagLayout());

    jPanel2.setOpaque(false);
    titleLabel.setIcon(new javax.swing.ImageIcon(getClass().getResource("/resources/tume1.gif")));
    titleLabel.setText("\u30c7\u30fc\u30bf\u30d9\u30fc\u30b9");
    jPanel2.add(titleLabel, new java.awt.GridBagConstraints());

    add(jPanel2, java.awt.BorderLayout.WEST);

    jPanel3.setLayout(new java.awt.GridBagLayout());

    jPanel3.setOpaque(false);
    buttonPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.CENTER, 1, 0));

    buttonPanel.setOpaque(false);
    panelCloseButton1.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
    buttonPanel.add(panelCloseButton1);

    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 2);
    jPanel3.add(buttonPanel, gridBagConstraints);

    add(jPanel3, java.awt.BorderLayout.EAST);

  }// </editor-fold>//GEN-END:initComponents
  
  
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JPanel buttonPanel;
  private javax.swing.JPanel jPanel2;
  private javax.swing.JPanel jPanel3;
  private to.tetramorph.starbase.widget.PanelCloseButton panelCloseButton1;
  private javax.swing.JLabel titleLabel;
  // End of variables declaration//GEN-END:variables
  
}
