/*
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */
package to.tetramorph.util;

import java.awt.Image;
import java.awt.Toolkit;
import java.net.URL;
import javax.swing.ImageIcon;

/**
 * 画像ファイルを読み込みImageIconやImageオブジェクトを作成する。
 * 画像ファイルはクラスパスの位置がルート。つまりclassファイルの置き場所を"/"と
 * する。一般的に"/resources/images"といったフォルダを用意し、そこにまとめて画像
 * ファイルを置く。classファイルはjar書庫にパックされるが、そのとき/resourcesも
 * 一緒に同梱する。そうしておけばこのクラスのメソッドは、自動的にjar書庫から、
 * 指定された画像ファイルをロードする。
 */
public class IconLoader {

    /**
     * 画像ファイルからImageIconを返す。画像ファイルにはgif,png,jpgなどが使用可。
     * @param path クラスパスをルートとみなしたファイルのパス。
     * "/resources/images/icon.gif"などと指定する。
     */
    public static ImageIcon getImageIcon(String path) {
        URL imgURL = IconLoader.class.getResource(path);
        if (imgURL != null) {
            return new ImageIcon(imgURL);
        } else {
            //System.err.println("Couldn't find file: " + path);
            return null;
        }
    }

    /**
     * 画像ファイルからImageを返す。画像ファイルにはgif,png,jpgなどが使用可。
     * @param path クラスパスをルートとみなしたファイルのパス。
     * "/resources/images/icon.gif"などと指定する。
     */
    public static Image getImage(String path) {
        URL imgURL = IconLoader.class.getResource(path);
        if (imgURL != null) {
            return Toolkit.getDefaultToolkit().createImage(imgURL);
        } else {
            //System.err.println("Couldn't find file: " + path);
            return null;
        }

    }
}
