/*
 * LoggerOutputStream.java
 * Created on 2011/08/08, 19:40:00.
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */
package to.tetramorph.util;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;

/**
 * System.outやSystem.errの出力を、コンソールだけではなくファイルにも出力できる
 * ようにするストリーム。
 * 次のように使う。
 * <pre>
 * エラーと標準出力を別々にファイル出力したいとき。
 * System.setOut(new PrintStream(new LoggerOutputStream( log_file,System.out)));
 * System.setErr(new PrintStream(new LoggerOutputStream( err_file,System.out)));
 * エラーと標準出力をまとめて一本のファイルに記録したいとき。
 * PrintStream ps = new PrintStream(new LoggerOutputStream( log_file,System.out));
 * System.setOut(ps);
 * System.setErr(ps);
 * </pre>
 * なおスレッドセーフではない。スレッドセーフであるべきだが、それを実装しようと
 * するとこういうやり方ではそもそも無理がありすぎる。
 * @author 大澤義孝
 */
public class LoggerOutputStream extends OutputStream {
    FileOutputStream stream;
    PrintStream ps;
    public LoggerOutputStream( File logFile, PrintStream ps ) throws IOException {
        stream = new FileOutputStream(logFile);
        this.ps = ps;
    }

    @Override
    public void write(int b) throws IOException {
        try {
        stream.write(b);
        ps.write(b);
        } catch ( IOException e ) {
        }
    }

    @Override
    public void close() {
        try {
            stream.flush();
            stream.close();
            ps.close();
            super.close();
        } catch ( IOException e ) {
        }
    }
}
