/*
 * MaximumWindowBounds.java
 *
 * Created on 2006/09/12, 0:56
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.util;

import java.awt.Frame;
import java.awt.GraphicsEnvironment;
import java.awt.Rectangle;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/**
 * フレームが最大化されたときツールバー領域にかさならないよう最適なサイズで
 * 最大化させる。
 * <pre>
 * class Hoge extends JFrame {
 *   ....
 *   addWindowFocusListener(new MaximumWindowBounds(this));
 * }
 * </pre>
 * @author 大澤義孝
 */
public class MaximumWindowBounds extends WindowAdapter {

    private Frame frame;

    /**
     * Frame#addWindowFocusListener(..)でこのクラスのインスタンスを登録して使う。
     * 最適な最大化をサポートさせたいframeを指定する。
     */
    public MaximumWindowBounds(Frame frame) {
        this.frame = frame;
    }

    /**
     * コンストラクタで指定されたFrameにフォーカスが移ったときにイベント処理として
     * 呼び出される。
     */
    @Override
    public void windowGainedFocus(WindowEvent e) {
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        //タスクバーを除外した画面サイズを取得
        Rectangle rect = ge.getMaximumWindowBounds();
        frame.setMaximizedBounds(rect);
    }
}
