/*
 * LanguageDialog.java
 *
 * Created on 2008/03/20, 21:44
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.widget;

import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Locale;
import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;
import javax.swing.UIManager;
import javax.swing.event.ListDataListener;
import to.tetramorph.util.ParentWindow;

/**
 * 言語を選択するためのダイアログ。
 * @author  大澤義鷹
 */
public class LanguageDialog extends JDialog {

    //現在選択されているロケールが格納される
    Locale selectedLocale;

    //リストモデルが使用するLocaleのリスト
    List<Locale> localeList = new ArrayList<Locale>();

    private LanguageDialog( Frame parent ) {
        this( parent, null );
    }

    /**
     * Creates new form LanguageDialog
     */
    private LanguageDialog( Frame parent, Locale defaultLocale ) {
        super( parent, true );
        initComponents();
        getRootPane().setDefaultButton( acceptButton );
        ParentWindow.setEscCloseOperation( this, new AbstractAction() {
            @Override
            public void actionPerformed(ActionEvent evt) {
                abortButton.doClick();
            }
        });
        //localListの重複している言語ｺｰﾄﾞをﾕﾆｰｸにして取り出す
        Locale [] locales = Locale.getAvailableLocales();
        HashSet<String> hash = new LinkedHashSet<String>();
        for ( Locale s : locales ) hash.add( s.getLanguage() );
        Iterator ite = hash.iterator();
        localeList = new ArrayList<Locale>();
        while ( ite.hasNext() ) {
            String lang = (String)ite.next();
            Locale locale = new Locale( lang );
            localeList.add( locale );
        }
        jList1.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        jList1.setModel( new LocaleListModel() );
        jList1.addMouseListener( new SelectionListener() );
        selectionLocale( defaultLocale );
        acceptButton.setEnabled( true );
        this.setLocationRelativeTo(parent);
        //ダイアログが可視化されるのを待って、選択行が見えるようにスクロールする

        new Thread( new Runnable() {
            @Override
            public void run() {
                while ( ! LanguageDialog.this.isVisible() ) {
                    try { Thread.sleep( 5 );
                    } catch ( InterruptedException e ) {}
                }
                scroll();
            }
        }).start();
    }

    /**
     * 指定されたロケールと同じ言語名をJListの中で選択する。
     * 無事に選択できたときはフィールドselectedLocaleに選択されたlocale(これは
     * たいがい引数で指定したlocaleと同じ)がセットされる。
     */
    private void selectionLocale( Locale locale ) {
        if ( locale == null ) {
            locale = Locale.getDefault();
        }
        String argLang = locale.getLanguage();
        for ( int i=0; i < localeList.size(); i++ ) {
            String lang = localeList.get(i).getLanguage();
            if ( argLang.equals( lang ) ) {
                jList1.setSelectedIndex( i );
                selectedLocale = locale;
                return;
            }
        }
        // まずあり得ないが、選択できなかったときは最初のものを選択
        selectedLocale = localeList.get(0);
        jList1.setSelectedIndex( 0 );
    }

    /**
     * 指定されたインデックスまでJListをスクロールし、選択が見えるようにする。
     */
    public void scroll() {
        int index = jList1.getSelectedIndex();
        if ( index < 0 ) return;
        JScrollBar bar = scrollPane.getVerticalScrollBar();
        int max = bar.getMaximum();
        int h = max / localeList.size();
        int height = scrollPane.getViewport().getExtentSize().height;
        int rows = height /h;
        bar.setValue( index * h - ( rows / 2 * h ) );
    }

    /**
     * 選択されたロケールを返す。選択中止の場合はnullを返す。
     */
    private Locale getSelectedLocale() {
        return selectedLocale;
    }

    /**
     * JListのリストモデル
     */
    class LocaleListModel implements ListModel {

        @Override
        public int getSize() {
            return localeList.size();
        }

        @Override
        public Object getElementAt( int index ) {
            return localeList.get( index ).getDisplayLanguage();
        }
        @Override
        public void addListDataListener(ListDataListener l) {
        }
        @Override
        public void removeListDataListener(ListDataListener l) {
        }
    }

    /**
     * マウスでリストを選択したことを検出するリスナ
     */
    class SelectionListener extends MouseAdapter {
        @Override
        public void mouseClicked( MouseEvent evt ) {
            int index = jList1.getSelectedIndex();
            if ( index < 0 ) {
                selectedLocale = null;
                acceptButton.setEnabled( false );
                return;
            }
            acceptButton.setEnabled( true );
            selectedLocale = localeList.get(index);
            if ( evt.getClickCount() == 2 )
                dispose();
        }
    }

    /**
     * 言語選択ダイアログを開く。システムのデフォルトのロケールに従って選択状態が
     * 設定される。
     * @param parent 親となるフレーム
     */
    public static final Locale showDialog( Frame parent ) {
        return showDialog( parent, null );
    }

    /**
     * 言語選択ダイアログを開く
     * @param parent 親となるフレーム
     * @param defaultLocale 最初に選択状態にするLocale
     */
    public static final Locale showDialog( Frame parent,
                                              Locale defaultLocale ) {
        LanguageDialog ld = new LanguageDialog( parent, defaultLocale );
        ld.setVisible( true );
        return ld.getSelectedLocale();
    }

    /**
     * mainから呼ばれる。テストプログラムの一部。
     */
    private static void createAndShowGUI() {
        if ( UIManager.getLookAndFeel().getName().equals("Metal")) {
            UIManager.put("swing.boldMetal", Boolean.FALSE);
            Toolkit.getDefaultToolkit().setDynamicLayout(true);
        }
        final JFrame frame = new JFrame("LanguageDialog Test");
        JButton button = new JButton("TEST");
        JPanel pane = new JPanel(new FlowLayout());
        pane.add(button);
        ActionListener listener = new ActionListener() {
            //TESTボタンが押されたら、ダイアログを開く
            @Override
            public void actionPerformed(ActionEvent e) {
//                Locale loc = showDialog( frame, new Locale("ja") );
                Locale loc = showDialog( frame, null );
                System.out.println("selected locale = " + loc );
            }
        };
        button.addActionListener(listener);
        frame.getRootPane().setDefaultButton(button);
        frame.add(pane);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.pack();
        frame.setVisible(true);
    }
    /**
     * このクラスのデモを行う。テスト用。引数は不要。
     */
    public static void main(String[] args) {
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                createAndShowGUI();
            }
        });
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        javax.swing.JPanel jPanel1;
        javax.swing.JPanel jPanel2;
        to.tetramorph.widget.ResizeIconPanel resizeIconPanel1;

        scrollPane = new javax.swing.JScrollPane();
        jList1 = new javax.swing.JList();
        jPanel1 = new javax.swing.JPanel();
        resizeIconPanel1 = new to.tetramorph.widget.ResizeIconPanel();
        jPanel2 = new javax.swing.JPanel();
        acceptButton = new javax.swing.JButton();
        abortButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("\u8a00\u8a9e\u306e\u9078\u629e");
        setMinimumSize(new java.awt.Dimension(170, 100));
        jList1.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 4, 1, 4));
        jList1.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "Item 1", "Item 2", "Item 3", "Item 4", "Item 5" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        scrollPane.setViewportView(jList1);

        getContentPane().add(scrollPane, java.awt.BorderLayout.CENTER);

        jPanel1.setLayout(new java.awt.BorderLayout());

        jPanel1.add(resizeIconPanel1, java.awt.BorderLayout.EAST);

        acceptButton.setText("\u6c7a\u5b9a");
        acceptButton.setEnabled(false);
        acceptButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                acceptButtonActionPerformed(evt);
            }
        });

        jPanel2.add(acceptButton);

        abortButton.setText("\u4e2d\u6b62");
        abortButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                abortButtonActionPerformed(evt);
            }
        });

        jPanel2.add(abortButton);

        jPanel1.add(jPanel2, java.awt.BorderLayout.CENTER);

        getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void acceptButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_acceptButtonActionPerformed
        int index = jList1.getSelectedIndex();
        selectedLocale = localeList.get(index);
        dispose();
    }//GEN-LAST:event_acceptButtonActionPerformed

    private void abortButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_abortButtonActionPerformed
        selectedLocale = null;
        dispose();
    }//GEN-LAST:event_abortButtonActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton abortButton;
    private javax.swing.JButton acceptButton;
    private javax.swing.JList jList1;
    private javax.swing.JScrollPane scrollPane;
    // End of variables declaration//GEN-END:variables

}
