/*
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */
package to.tetramorph.tzdialog;

import java.awt.Component;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Window;
import java.awt.event.KeyEvent;
import javax.swing.Action;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;
/**
 * Dialogを作るときにかかせない親となるDialogまたはFrameを見つけ出すツール。
 */
class ParentWindow {
    private ParentWindow() { }
    /**
     * 与えられたコンポーネントの親を辿り、最初にみつけたFrameまたはDialog
     * をWindowオブジェクトで返す。
     * @param parentComponent 探索を開始するコンポーネント
     * @return java.awt.Dialogまたはjava.awt.FrameのWindowオブジェクト
     */
    public static Window getWindowForComponent(Component parentComponent) {
        if(parentComponent == null) return JOptionPane.getRootFrame();
        if(parentComponent instanceof Frame || parentComponent instanceof Dialog)
            return (Window)parentComponent;
        return getWindowForComponent(parentComponent.getParent());
    }
    
    /**
     * ESCキーでダイアログを閉じるためのアクションを登録する。
     * アクションは次のようにして作成し登録する。
     * <pre>
     * AbstractAction escCloser = new AbstractAction("了解") {
     *    public void actionPerformed(ActionEvent evt) {
     *        acceptButton.doClick();
     *    }
     * };
     * ParentWindow.setEscCloseOperation( owner, escCloser );
     * </pre>
     * @param owner 親ダイアログ
     * @param action ESCキーが押されたときに実行するアクション
     */
    public static void setEscCloseOperation( JDialog owner, Action action) {
        InputMap imap = owner.getRootPane()
            .getInputMap( JComponent.WHEN_IN_FOCUSED_WINDOW );
        imap.put( KeyStroke.getKeyStroke( KeyEvent.VK_ESCAPE, 0 ), Action.NAME);
        owner.getRootPane().getActionMap().put( Action.NAME, action );
    }
    
    /**
     * ESCキーでダイアログを閉じるためのアクションを登録する。
     * アクションは次のようにして作成し登録する。
     * <pre>
     * AbstractAction escCloser = new AbstractAction("了解") {
     *    public void actionPerformed(ActionEvent evt) {
     *        acceptButton.doClick();
     *    }
     * };
     * ParentWindow.setEscCloseOperation( owner, escCloser );
     * </pre>
     * @param owner 親フレーム
     * @param action ESCキーが押されたときに実行するアクション
     */
    public static void setEscCloseOperation( JFrame owner, Action action) {
        InputMap imap = owner.getRootPane()
            .getInputMap( JComponent.WHEN_IN_FOCUSED_WINDOW );
        imap.put( KeyStroke.getKeyStroke( KeyEvent.VK_ESCAPE, 0 ), Action.NAME);
        owner.getRootPane().getActionMap().put( Action.NAME, action );
    }
}
