/*
 * TimeZoneDialog.java
 *
 * Created on 2006/05/19, 7:50
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */
package to.tetramorph.tzdialog;

import java.awt.Component;
import java.awt.Dialog;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.TimeZone;
import javax.swing.table.AbstractTableModel;
import static java.lang.System.*;
import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

/**
 * タイムゾーン選択用ダイアログ。Javaがサポートする全タイムゾーンのリストを
 * 表示して、その中からユーザに任意のタイムゾーンを選択させることができる。
 * @author 大澤義鷹
 */
public class TimeZoneDialog extends javax.swing.JDialog {
    //テスト用JFrame

    private static JFrame frame = new JFrame();
    //全タイムゾーンのリスト
    private java.util.List<String[]> zoneList = new ArrayList<String[]>();
    //選択されたタイムゾーンの保管用
    private static String zone_id = null;
    //テーブルヘッダ名
    private static final String[] headArray = {
        "ID", "ゾーン名", "時差", "夏時間" };
    //コンストラクタから呼び出される初期化メソッド

    private void init() {
//    setMinimumSize(new Dimension(356,200));
//    setPreferredSize(new Dimension(356,300));
        setZoneList();
        initComponents();
        zoneTable.setModel(new AbstractTableModel() {
            //テーブルの列数を返す(それは列名配列の長さと同じ)

            @Override
            public int getColumnCount() {
                return 4;
            }
            //テーブルの行数を返す(それはdata配列の長さと同じ)

            @Override
            public int getRowCount() {
                return zoneList.size();
            }
            //行・列指定で、data配列の参照を返す。

            @Override
            public Object getValueAt(int row, int col) {
                return zoneList.get(row)[col];
            }
            //列数指定で列名を返す。

            @Override
            public String getColumnName(int column) {
                return headArray[column];
            }
            //列数指定でその列に入っているオブジェクトのクラスを返す

            @Override
            public Class getColumnClass(int c) {
                return getValueAt(0, c).getClass();
            }
            //セルが編集できるかどうかを返す

            @Override
            public boolean isCellEditable(int row, int col) {
                return false;
            }
            //行・列指定で指定のオブジェクトをdata配列にセットする。

            @Override
            public void setValueAt(Object aValue, int row, int column) {
            }
        });
        //this.addKeyListener(new KeyHandler());
        ParentWindow.setEscCloseOperation(this, new AbstractAction() {

            @Override
            public void actionPerformed(ActionEvent evt) {
                cancelButton.doClick();
            }
        });
        SwingUtilities.invokeLater(new Runnable() {

            @Override
            public void run() {
                TableColumnResizer.adjust(zoneTable);
                //revalidate();
            }
        });
        pack();
    }

    /**
     * このコンストラクタはshowDialogメソッドから呼び出される。
     */
    private TimeZoneDialog(java.awt.Frame parent) {
        super(parent, true);
        init();
    }

    private TimeZoneDialog(java.awt.Dialog parent) {
        super(parent, true);
        init();
    }

    /**
     * zoneListにタイムゾーンの全リストを格納する。
     * 一つのデータは要素が3つのString配列で、
     * [0]=id
     * [1]=ゾーン名
     * [2]=時差("9.0"等実数を表す文字列)
     */
    private void setZoneList() {
        String[] list = TimeZone.getAvailableIDs(); //有効なIDを取得
        zoneList = new ArrayList<String[]>();
        for (int i = 0; i < list.length; i++) {
            TimeZone tz = TimeZone.getTimeZone(list[i]);
            String[] field = new String[4];
            field[0] = list[i];
            field[1] = tz.getDisplayName(false, TimeZone.LONG);
            field[2] = String.format( "%2.1f",
                                       tz.getRawOffset() / 3600 / 1000f);
            field[3] = tz.useDaylightTime() ? "有" : "無";
            zoneList.add(field);
        }
    }

    /**
     * 「決定」処理。
     * 決定ボタン、テーブル上でのダブクリ、テーブル上での改行入力のアクション処理
     * からこのメソッドは呼び出される。
     * 選択された項目番号から、ゾーンIDを得てそれをstaticの変数にコピーしたら
     * dispose()する。
     */
    private void accept() {
        int i = zoneTable.getSelectedRow();
        if (i < 0) {
            resultLabel.setText("<html><font color=red>データ未選択</font>");
            return;
        }
        String[] value = zoneList.get(i);
        zone_id = value[0];
        dispose();
    }

    /**
     * タイムゾーン選択ダイアログを開いて、タイムゾーン選択をユーザに促し、
     * 選択結果を返す。選択が中止された場合はnullを返す。
     * @param parent 呼び出し元のComponent。
     * @return java.util.TimeZone
     */
    public static TimeZone showDialog(Component parent) {
        Window window = ParentWindow.getWindowForComponent(parent);
        //SwingUtilities.getWindowAncestor(parent);
        final TimeZoneDialog dialog;
        if (window instanceof Frame) {
            dialog = new TimeZoneDialog((Frame) window);
        } else {
            dialog = new TimeZoneDialog((Dialog) window);
        }
        dialog.setComponentOrientation(window.getComponentOrientation());
        dialog.pack();
        //親コンポーネントに対してセンタリング
        dialog.setLocationRelativeTo(window);
        dialog.setVisible(true);
        if (zone_id == null) {
            return null;
        }
        return TimeZone.getTimeZone(zone_id);
    }
//  /**
//   * このコンポーネントのフォーカスを受け取る部品にキーリスナを登録する
//   */
//  public void addKeyListener(KeyListener l) {
//    super.addKeyListener(l);
//    cancelButton.addKeyListener(l);
//    acceptButton.addKeyListener(l);
//    allButton.addKeyListener(l);
//    placeSearchButton.addKeyListener(l);
//    placeTextField.addKeyListener(l);
//    timeSearchButton.addKeyListener(l);
//    timeTextField.addKeyListener(l);
//    zoneTable.addKeyListener(l);
//  }
//  //ESCキーでダイアログをクローズ
//  private class KeyHandler extends KeyAdapter {
//    public void keyTyped(KeyEvent e) {
//      int code =(int)e.getKeyChar();
//      if(code == KeyEvent.VK_ESCAPE) { //ESCキーでウィンドウ閉じる
//        cancelButton.doClick();
//      }
//    }
//  }

    /**
     * mainから呼ばれる。テストプログラムの一部。
     */
    private static void createAndShowGUI() {
        if (UIManager.getLookAndFeel().getName().equals("Metal")) {
//            UIManager.put("swing.boldMetal", Boolean.FALSE);
//            try {
//                UIManager.setLookAndFeel(
//                        "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel");
//            } catch (Exception e) {
//                e.printStackTrace();
//            }
            JDialog.setDefaultLookAndFeelDecorated(true);
            JFrame.setDefaultLookAndFeelDecorated(true);
            Toolkit.getDefaultToolkit().setDynamicLayout(true);
        }

        JButton button = new JButton("TEST");
        JPanel pane = new JPanel(new FlowLayout());
        pane.add(button);
        ActionListener listener = new ActionListener() {
            //TESTボタンが押されたら、ダイアログを開く

            @Override
            public void actionPerformed(ActionEvent e) {
                TimeZone tz = TimeZoneDialog.showDialog(frame);
                if (tz == null) {
                    out.println("選択は中止されました");
                } else {
                    out.println(tz.getDisplayName() + "が選択されました。");
                }
            }
        };
        button.addActionListener(listener);
        frame.getRootPane().setDefaultButton(button);
        frame.add(pane);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.pack();
        frame.setVisible(true);
    }

    /**
     * このクラスのデモを行う。テスト用。引数は不要。
     */
    public static void main(String[] args) {
        javax.swing.SwingUtilities.invokeLater( new Runnable() {
            @Override
            public void run() {
                createAndShowGUI();
            }
        });
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        javax.swing.JPanel jPanel3 = new javax.swing.JPanel();
        javax.swing.JPanel jPanel2 = new javax.swing.JPanel();
        placeSearchButton = new javax.swing.JButton();
        timeSearchButton = new javax.swing.JButton();
        placeTextField = new javax.swing.JTextField();
        timeTextField = new javax.swing.JTextField();
        javax.swing.JLabel jLabel1 = new javax.swing.JLabel();
        javax.swing.JLabel jLabel2 = new javax.swing.JLabel();
        allButton = new javax.swing.JButton();
        resultLabel = new javax.swing.JLabel();
        javax.swing.JScrollPane jScrollPane1 = new javax.swing.JScrollPane();
        zoneTable = new javax.swing.JTable();
        javax.swing.JPanel jPanel1 = new javax.swing.JPanel();
        acceptButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("タイムゾーンの選択");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        jPanel3.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        jPanel2.setLayout(new java.awt.GridBagLayout());

        placeSearchButton.setMnemonic('S');
        placeSearchButton.setText("検索(S)");
        placeSearchButton.setToolTipText("キーワードで検索します");
        placeSearchButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                placeSearchButtonActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        jPanel2.add(placeSearchButton, gridBagConstraints);

        timeSearchButton.setMnemonic('F');
        timeSearchButton.setText("検索(F)");
        timeSearchButton.setToolTipText("時差で検索します");
        timeSearchButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                timeSearchButtonActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        jPanel2.add(timeSearchButton, gridBagConstraints);

        placeTextField.setColumns(8);
        placeTextField.setToolTipText("<html>IDとゾーン名に入力キーワードが含まれているデータを抽出</html>");
        placeTextField.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                placeTextFieldKeyPressed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        jPanel2.add(placeTextField, gridBagConstraints);

        timeTextField.setColumns(8);
        timeTextField.setToolTipText("入力された時差と同じ値を持つデータを抽出。<br>小数点入力も受け付けます。</html>");
        timeTextField.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                timeTextFieldKeyPressed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        jPanel2.add(timeTextField, gridBagConstraints);

        jLabel1.setText("ゾーンやID名");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        jPanel2.add(jLabel1, gridBagConstraints);

        jLabel2.setText("時差");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        jPanel2.add(jLabel2, gridBagConstraints);

        allButton.setMnemonic('L');
        allButton.setText("全リスト(L)");
        allButton.setToolTipText("タイムゾーンの全リストを表示");
        allButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                allButtonActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        jPanel2.add(allButton, gridBagConstraints);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        jPanel2.add(resultLabel, gridBagConstraints);

        jPanel3.add(jPanel2);

        getContentPane().add(jPanel3, java.awt.BorderLayout.NORTH);

        jScrollPane1.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                jScrollPane1KeyPressed(evt);
            }
        });

        zoneTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "タイトル 1", "タイトル 2", "タイトル 3", "タイトル 4"
            }
        ));
        zoneTable.setToolTipText("<html>ダブルクリックで決定<br>選択後改行キーで決定。</html>");
        zoneTable.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                zoneTableKeyPressed(evt);
            }
        });
        zoneTable.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                zoneTableMouseClicked(evt);
            }
        });
        jScrollPane1.setViewportView(zoneTable);

        getContentPane().add(jScrollPane1, java.awt.BorderLayout.CENTER);

        acceptButton.setMnemonic(KeyEvent.VK_ENTER);
        acceptButton.setMnemonic('Y');
        acceptButton.setText("決定(Y)");
        acceptButton.setToolTipText("");
        acceptButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                acceptButtonActionPerformed(evt);
            }
        });
        jPanel1.add(acceptButton);

        cancelButton.setMnemonic('N');
        cancelButton.setText("中止(N)");
        cancelButton.setToolTipText("");
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        });
        jPanel1.add(cancelButton);

        getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /** 決定ボタンのアクション */
    private void acceptButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_acceptButtonActionPerformed
        accept();
    }//GEN-LAST:event_acceptButtonActionPerformed
    /**
     * 時差入力用のテキストフィールドでEnterキーが押されたときのアクション。
     * 時差検索ボタンが押された事にする。
     */
    private void timeTextFieldKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_timeTextFieldKeyPressed
        if (evt.getKeyCode() == KeyEvent.VK_ENTER) {
            timeSearchButton.doClick();
        }
    }//GEN-LAST:event_timeTextFieldKeyPressed
    /** 一覧テーブル上でマウスがダブルクリックされたときのアクション */
    private void zoneTableMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_zoneTableMouseClicked
        resultLabel.setText("");
        if (evt.getClickCount() == 2) {
            accept();
        }
    }//GEN-LAST:event_zoneTableMouseClicked
    /**
     * 一覧テーブル上でEnterキーが押されたときのアクション。
     * 決定ボタンが押されたのと同じことにする。
     */
    private void zoneTableKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_zoneTableKeyPressed
        if (evt.getKeyCode() == KeyEvent.VK_ENTER) {
            acceptButton.doClick();
        }
    }//GEN-LAST:event_zoneTableKeyPressed

    private void jScrollPane1KeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jScrollPane1KeyPressed
// TODO add your handling code here:
    }//GEN-LAST:event_jScrollPane1KeyPressed
    /**
     * ゾーン名やID名のテキストフィールドでEnterキーが押されたときのアクション
     * ゾーン名検索ボタンを押したのと同じ事にする。
     */
    private void placeTextFieldKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_placeTextFieldKeyPressed
        if (evt.getKeyCode() == KeyEvent.VK_ENTER) {
            placeSearchButton.doClick();
        }
    }//GEN-LAST:event_placeTextFieldKeyPressed
    /**
     *全リストボタンが押されたときのアクション。テーブルにゾーンの全リストを表示。
     */
    private void allButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_allButtonActionPerformed
        setZoneList();
        zoneTable.revalidate();
    }//GEN-LAST:event_allButtonActionPerformed
    /**
     *時差の検索ボタンが押された時のアクション。
     */
    private void timeSearchButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_timeSearchButtonActionPerformed
        java.util.List<String[]> list = new ArrayList<String[]>();
        setZoneList();
        boolean err = false;
        float timef = 0f;
        try {
            timef = Float.parseFloat(timeTextField.getText());
        } catch (NumberFormatException e) {
            resultLabel.setText("入力エラー");
            timeTextField.setText("");
            err = true;
        }
        if (!err) {
            for (int i = 0; i < zoneList.size(); i++) {
                if (Float.parseFloat(zoneList.get(i)[2]) == timef) {
                    list.add(zoneList.get(i));
                    continue;
                }
            }
            zoneList = list;
            resultLabel.setText("該当 " + list.size() + "件");
        }
        zoneTable.revalidate();
    }//GEN-LAST:event_timeSearchButtonActionPerformed
    /**
     *ゾーン名の検索ボタンが押されたときのアクション
     */
    private void placeSearchButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_placeSearchButtonActionPerformed
        java.util.List<String[]> list = new ArrayList<String[]>();
        setZoneList();
        String txt = placeTextField.getText().toLowerCase();
        for (int i = 0; i < zoneList.size(); i++) {
            if (zoneList.get(i)[0].toLowerCase().indexOf(txt) >= 0 || zoneList.get(i)[1].toLowerCase().indexOf(txt) >= 0) {
                list.add(zoneList.get(i));
                continue;
            }
        }
        zoneList = list;
        zoneTable.revalidate();
        resultLabel.setText("該当 " + list.size() + "件");
    }//GEN-LAST:event_placeSearchButtonActionPerformed
    /**
     *中止ボタンのアクション。zone_idにnullをセットしてdispose();
     */
    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
        zone_id = null;
        dispose();
    }//GEN-LAST:event_cancelButtonActionPerformed
    /**
     *フレームの×ボタンが押されたときの処理。zone_idにnullをセット。
     */
    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        zone_id = null;
        //System.out.println("クローズボタン押されたよ");
    }//GEN-LAST:event_formWindowClosing
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton acceptButton;
    private javax.swing.JButton allButton;
    private javax.swing.JButton cancelButton;
    private javax.swing.JButton placeSearchButton;
    private javax.swing.JTextField placeTextField;
    private javax.swing.JLabel resultLabel;
    private javax.swing.JButton timeSearchButton;
    private javax.swing.JTextField timeTextField;
    private javax.swing.JTable zoneTable;
    // End of variables declaration//GEN-END:variables
}
